{ Copyright (C) 2012 H Page-Clark

  This source is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.

  This code is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.

  A copy of the GNU General Public License is available on the World Wide Web
  at <http://www.gnu.org/copyleft/gpl.html>. You can also obtain it by writing
  to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
  MA 02111-1307, USA. }

unit main_stream;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, StdCtrls,
  Spin, ComCtrls, demo_stream, BlowFish;

const savedFilename: TFilename = 'stream.dat';

type

  { TForm1 }

  TForm1 = class(TForm)
    btnDecrypt: TButton;
    btnEncrypt: TButton;
    btnSaveStream: TButton;
    btnLoadStream: TButton;
    btnGenerateStream: TButton;
    edtRecord: TEdit;
    lblAdjustPosition: TLabel;
    lblNoOfRecords: TLabel;
    lblPositionDesc: TLabel;
    lblSavedFileSize: TLabel;
    lblOrigin: TLabel;
    lblStreamSize: TLabel;
    lblViewRecord: TLabel;
    memoDisplay: TMemo;
    memoEncrypted: TMemo;
    memoDecrypted: TMemo;
    seNoOfRecords: TSpinEdit;
    seCurrentRecNo: TSpinEdit;
    tbPosition: TTrackBar;
    udMover: TUpDown;
    procedure btnDecryptClick(Sender: TObject);
    procedure btnEncryptClick(Sender: TObject);
    procedure btnGenerateStreamClick(Sender: TObject);
    procedure btnLoadStreamClick(Sender: TObject);
    procedure btnSaveStreamClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure seCurrentRecNoChange(Sender: TObject);
    procedure udMoverClick(Sender: TObject; Button: TUDBtnType);
  private
    ds: TdemoStream;
    sOriginal, sEncrypted: TStringStream;
    key: string;
  public
    procedure DisplayStream;
  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

{ TForm1 }

procedure TForm1.btnGenerateStreamClick(Sender: TObject);
begin
  ds.Free;
  ds:= TdemoStream.Create(seNoOfRecords.Value);
  seCurrentRecNo.MaxValue:= seNoOfRecords.Value-1;
  DisplayStream;
  seCurrentRecNoChange(nil);
  btnSaveStream.Enabled:= True;
  lblSavedFileSize.Caption:= EmptyStr;
  memoDecrypted.Lines.Clear;
  memoEncrypted.Lines.Clear;
end;

procedure TForm1.btnEncryptClick(Sender: TObject);
var be: TBlowFishEncryptStream;
begin
  if memoDisplay.Lines.Count = 0 then Exit;
  key:= 'example cipher key for Blowfish';
  sOriginal:= TStringStream.Create(EmptyStr);
  be:= TBlowFishEncryptStream.Create(key, sOriginal);
  be.WriteAnsiString(memoDisplay.Text);
  be.Free;
  memoEncrypted.Text:= sOriginal.DataString;
end;

procedure TForm1.btnDecryptClick(Sender: TObject);
var bd: TBlowFishDeCryptStream;
begin
  sEncrypted:= TStringStream.Create(sOriginal.DataString);
  bd:= TBlowFishDeCryptStream.Create(key, sEncrypted);
  memoDecrypted.Text:= bd.ReadAnsiString;
  bd.Free;
  sEncrypted.Free;
  sOriginal.Free;
end;

procedure TForm1.btnLoadStreamClick(Sender: TObject);
begin
  FreeAndNil(ds);
  ds := TdemoStream.Create(0);
  ds.LoadFromFile(savedFilename);
  seNoOfRecords.Value:= ds.Size div SizeOf(ds.rec);
  seCurrentRecNo.MaxValue:= seNoOfRecords.Value-1;
  DisplayStream;
  btnLoadStream.Enabled:= False;
  seCurrentRecNoChange(nil);
  memoDecrypted.Lines.Clear;
  memoEncrypted.Lines.Clear;
end;

procedure TForm1.btnSaveStreamClick(Sender: TObject);
begin
  if Assigned(ds) then
   begin
     memoDisplay.Lines.Clear;
     edtRecord.Text:= EmptyStr;
     ds.SaveToFile(savedFilename);
     lblSavedFileSize.Caption:= Format('Saved file size is: %d bytes',[FileSize(savedFilename)]);
     btnLoadStream.Enabled:= True;
     btnSaveStream.Enabled:= False;
     FreeAndNil(ds);
   end;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  ds.Free;
end;

procedure TForm1.seCurrentRecNoChange(Sender: TObject);
var l: integer;
begin
  if not Assigned(ds)then Exit;
  ds.Seek(seCurrentRecNo.Value*SizeOf(ds.rec), soFromBeginning);
  edtRecord.Text:= ds.RecAsString;
  l := Length(edtRecord.Text) + Length(LineEnding);
  tbPosition.Position:= tbPosition.Max*ds.Position div ds.Size;
  memoDisplay.SelStart:= (tbPosition.Position)*l;
  memoDisplay.SelLength:= l;
end;

procedure TForm1.udMoverClick(Sender: TObject; Button: TUDBtnType);
begin
  if not Assigned(ds)then Exit;
  case Button of
   btPrev: if (ds.Position >= SizeOf(ds.rec)) then
            begin
              ds.Seek(- 2*SizeOf(ds.rec), soFromCurrent);
              if (ds.Position < 0) then ds.Position:= 0;
              edtRecord.Text:= ds.RecAsString;
              seCurrentRecNo.Value:= seCurrentRecNo.Value-1;
            end;
   btNext: if (ds.Position < ds.Size) then
            begin
              edtRecord.Text:= ds.RecAsString;
              seCurrentRecNo.Value:= seCurrentRecNo.Value+1;
            end;
  end;
  seCurrentRecNoChange(nil);
end;

procedure TForm1.DisplayStream;
begin
  lblStreamSize.Caption:= Format('%d  ',[ds.Size]);
  tbPosition.Max:= ds.Size div SizeOf(ds.rec) - 1;
  memoDisplay.Lines.BeginUpdate;
    memoDisplay.Clear;
    ds.Position:= 0;
    while (ds.Position < ds.Size) do
     begin
      memoDisplay.Lines.Add(ds.RecAsString);
      tbPosition.Position:= (ds.Size * seNoOfRecords.Value) div ds.Position;
     end;
  memoDisplay.Lines.EndUpdate;
  seCurrentRecNo.MaxValue:= seNoOfRecords.Value;
  ds.Seek(seCurrentRecNo.Value * SizeOf(ds.rec), soFromBeginning);
  edtRecord.Text:= ds.RecAsString;
  tbPosition.Position:= (ds.Size * tbPosition.Max) div ds.Position;
end;

end.
